# -*- coding: utf-8 -*-
"""
 A plugin to organize audio-podcast
"""

import sys, os, os.path, xbmcaddon
import xbmc, xbmcgui, xbmcplugin
import urllib, re, time
#import hashlib
from shutil import rmtree, copy
from xml.dom import minidom
import traceback
from pprint import pprint

class RssFeedItem:
  def __init__(self, title, url, readed,filepath):
    self.title = title
    self.url = url
    self.readed = readed
    self.filepath = filepath
    
class RssFeed:
  def __init__(self, url):
    try:
      fileName = url.replace("http://","").replace("/","_");
      self.archiveFilePath = os.path.join(DIR_ARCHIVES, fileName);
      archiveFile = open(self.archiveFilePath,"r+");
      self.archiveContent = archiveFile.read();
      archiveFile.close();
      
      safe_url = url.replace( " ", "%20" );
      log('Downloading from url=%s' % safe_url);
      sock = urllib.urlopen( safe_url );
      doc = sock.read();
      sock.close();
      self.rssDoc = minidom.parseString(doc);
    except:
      errorOK();
  
  def getRssFeedItems(self):
    items = []
    for node in self.rssDoc.getElementsByTagName("item"):
      title = node.getElementsByTagName("title")[0].childNodes[0].data
      link = node.getElementsByTagName("enclosure")[0].getAttribute("url")
      readed = self.isReaded(link)
      items.append(RssFeedItem(title,link,readed,self.archiveFilePath))
    return items
  
  def countUnreadedItems(self):
    unreaded = 0
    for node in self.rssDoc.getElementsByTagName("item"):
      link = node.getElementsByTagName("enclosure")[0].getAttribute("url")
      if(not self.isReaded(link)):
	unreaded += 1
    return unreaded
  
  def isReaded(self,url):
    regEx = re.compile(url)
    return regEx.search(self.archiveContent)
  
  def markRead(self):
    archiveFile = open(self.archiveFilePath,"w")
    for item in self.getRssFeedItems():
      archiveFile.write(item.url+"\n")
    archiveFile.close()
      

  
  
  
__plugin__ = "AudioPodcast"
__version__ = '0.0.1'
__author__ = 'Raptor 2101 [raptor2101@gmx.com]'
__date__ = '06-09-2010'
__svn__ = 'http://xbmc-addons.googlecode.com/svn/tags/plugins/music/' + __plugin__

def log(msg):
  if type(msg) not in (str, unicode):
    xbmc.output("[%s]: %s" % (__plugin__, type(msg)))
    pprint (msg)
  else:
    xbmc.output("[%s]: %s" % (__plugin__, msg))

def errorOK(title="", msg=""):
  e = str( sys.exc_info()[ 1 ] )
  log(e)
  if not title:
    title = __plugin__
  if not msg:
    msg = "ERROR!"
  xbmcgui.Dialog().ok( title, msg, e )

def addDirectoryItem(name, url, mode, image=""):
  url = urllib.quote_plus(url);
  liz=xbmcgui.ListItem(name, "")
  liz.addContextMenuItems([(__language__(1020),"XBMC.RunPlugin(%s?url=%s&mode=%d)"%(sys.argv[0], url, mode+2),)])
  infoLabels = {"Title": name }
  
  liz.setInfo( "Music", infoLabels )
  #u = "%s?url=%s&mode=%s&name=%s" % (sys.argv[0], urllib.quote_plus(url), mode, urllib.quote_plus(name), )
  u = "%s?url=%s&mode=%s&name=%s" % (sys.argv[0], url, mode, urllib.quote_plus(name.encode('utf-8')), )
  log("%s" % u)
  return xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]),url=u,listitem=liz,isFolder=True)

def addStreamItem(feedItem):
  mode = -1
  if(feedItem.readed):
    title = feedItem.title
  else:
    title = "* "+feedItem.title
    
  liz=xbmcgui.ListItem( title, "")
  
  infoLabels = {"Title": title }
  liz.setInfo( "Music", infoLabels )
  liz.addContextMenuItems([(__language__(1020),"XBMC.RunPlugin(%s?url=%s&mode=4&filepath=%s)"%(sys.argv[0], urllib.quote_plus(feedItem.url), urllib.quote_plus(feedItem.filepath)),)])
  u = "%s?url=%s&mode=%s&name=%s&filepath=%s" % (sys.argv[0], urllib.quote_plus(feedItem.url), mode, urllib.quote_plus(title.encode('utf-8')), urllib.quote_plus(feedItem.filepath))
  return xbmcplugin.addDirectoryItem(handle=int(sys.argv[1]),url=u,listitem=liz,isFolder=False)

def get_params():
  """ extract params from argv[2] to make a dict (key=value) """
  paramDict = {}
  try:
    print "get_params() argv=", sys.argv
    if sys.argv[2]:
      paramPairs=sys.argv[2][1:].split( "&" )
      for paramsPair in paramPairs:
	paramSplits = paramsPair.split('=')
	if (len(paramSplits))==2:
	  paramDict[paramSplits[0]] = paramSplits[1]
  except:
    errorOK()
  return paramDict



def sumUnreadedElements(rootNode):
  sumUnreaded = 0
  for node in rootNode.childNodes:
    if not node.nodeName == "outline" : continue
    if node.hasChildNodes():
      sumUnreaded += sumUnreadedElements(node)
    else:
      xmlUrl = node.getAttribute("xmlUrl")
      xmlFeed = RssFeed(xmlUrl)
      sumUnreaded += xmlFeed.countUnreadedItems()
  return sumUnreaded;
      
def buildMenu(rootNode, rootUrl):
  for node in rootNode.childNodes:
    if not node.nodeName == "outline" : continue
    title = node.getAttribute('text')
    #unreaded = examineUnreadedElements(node)
    log(title)
    id = node.getAttribute('id')
    if node.hasChildNodes():
      unreadedItems = sumUnreadedElements(node)
      addDirectoryItem("(%d) %s"%(unreadedItems,title),rootUrl+"/"+id,0)
    else:
      xmlUrl = node.getAttribute("xmlUrl")
      xmlFeed = RssFeed(xmlUrl)
      addDirectoryItem("(%d) %s"%(xmlFeed.countUnreadedItems(),title),xmlUrl,1,"")


def findNode(node,rootUrl,url):  
  if url.startswith(rootUrl):
    if url == rootUrl:
      return node;
    else:
      for node in node.childNodes:
	if not node.nodeName == "outline" : continue
	id = node.getAttribute('id')
	if not url.startswith(rootUrl+"/"+id) : continue
	return findNode(node,rootUrl+"/"+id,url)


def recursiveMarkRead(node):
 for childNode in node.childNodes:
   if not childNode.nodeName == "outline" : continue
   if childNode.hasChildNodes():
     recursiveMarkRead(childNode)
   else:
     xmlUrl = node.getAttribute("xmlUrl")
     xmlFeed = RssFeed(xmlUrl)
     xmlFeed.markRead()
     

def displayRssFeed(url):
  xmlFeed = RssFeed(url)
  for rssFeedItem in xmlFeed.getRssFeedItems():
    addStreamItem(rssFeedItem)
  
def playStream(url,filepath):
  try:
      log("> playStream() " + url)
      plyr = xbmc.Player()
      plyr.play(str(url))
      isPlaying = plyr.isPlaying()
      fd = open(filepath,"r")
      content = fd.read();
      fd.close();
      content += url + "\n"
      fd = open(filepath,"w")
      fd.write(content)
      fd.close();
      xbmc.executebuiltin("Container.Refresh")
      return isPlaying
  except:
    errorOK("playStream()")
    return False

dialogProgress = xbmcgui.DialogProgress()

for number in sys.version_info:
  log("Version: %s" % number)
#######################################################################################################################    
# BEGIN !
#######################################################################################################################
try:
  __settings__ = xbmcaddon.Addon(id='plugin.audio.PodCatcher')
  __language__ = __settings__.getLocalizedString
  
  DIR_HOME = __settings__.getAddonInfo("profile")
  DIR_ARCHIVES = os.path.join(DIR_HOME, 'archives')
  PATH_FILE_OPML = os.path.join(DIR_HOME, 'opml.xml')
  
  log(PATH_FILE_OPML)
except:
  errorOK()

log(PATH_FILE_OPML)
xmldoc = minidom.parse(PATH_FILE_OPML)

params=get_params()

url=urllib.unquote_plus(params.get("url", ""))
name=urllib.unquote_plus(params.get("name",""))
mode=int(params.get("mode","0"))


if not url:
  url = "/"
  mode = 0

log("Mode: %s" % mode)
log("URL: %s" % url)
log("Name: %s" % name)

# itterate about the body-Tag, should only be one...
if mode == 0:
  for bodyNode in  xmldoc.getElementsByTagName('body'):
    node = findNode(bodyNode, "/", url)
    buildMenu(node, url);
  xbmcplugin.endOfDirectory(int(sys.argv[1]))
elif mode == 1:
  displayRssFeed(url)
  xbmcplugin.endOfDirectory(int(sys.argv[1]))
elif mode == 2:
  for bodyNode in  xmldoc.getElementsByTagName('body'):
    node = findNode(bodyNode, "/", url)
    recursiveMarkRead(node)
  xbmc.executebuiltin("Container.Refresh")
elif mode == 3:
  rssFeed = RssFeed(url)
  rssFeed.markRead()
  xbmc.executebuiltin("Container.Refresh")
elif mode == 4:
  filepath=urllib.unquote_plus(params.get("filepath",""))
  fd = open(filepath,"r")
  content = fd.read();
  fd.close();
  content += url + "\n"
  fd = open(filepath,"w")
  fd.write(content)
  fd.close();
  xbmc.executebuiltin("Container.Refresh")
elif mode == -1:
  filepath=urllib.unquote_plus(params.get("filepath",""))
  playStream(url,filepath)

